//	Varying variables
varying vec2 texCoord;	//	texCoords
varying vec3 normal;	//	normal
varying vec3 lightVec[4];		//	Light Vectors ( 1 - 4 Lights )

varying vec3 viewVec;	//	view Vector

//	Shared uniforms
uniform int numLights;		//	Number of lights

[Vertex Shader]

//	Transformation uniforms
uniform mat4 worldMatrix;		//	world matrix
uniform mat4 viewMatrix;		//	view/camera matrix
uniform mat4 projectionMatrix;	//	projection matrix

//	Lighting uniforms
uniform vec3 lightPositions[4];	//	Light positions

void main()
{
	//  Calculate World x View Matrix
	mat4 worldViewMatrix = viewMatrix * worldMatrix;
	//  Calculate Projection x WorldView Matrix
	mat4 worldViewProjectionMatrix = projectionMatrix * worldViewMatrix;
	//  Transform Vertex
	gl_Position = worldViewProjectionMatrix * gl_Vertex;
	//  Pass Texture coordinates to pixel shader
	texCoord = gl_MultiTexCoord0;
	
	//  Vertex World position
	vec3 worldPos = worldMatrix * gl_Vertex;
	
	//	Loop through all lights and calculate light vectors
	for( int i = 0; i < numLights; i++ )
	{
		//  Calculate Light[i] vector in world position
		lightVec[i] = normalize( lightPositions[i] - worldPos );
	}

	//  Calculate normal
	normal = normalize( (mat3)worldMatrix * gl_Normal );
	
	vec3 camPos = vec3( viewMatrix[3][0], viewMatrix[3][1], viewMatrix[3][2] );
	viewVec =  worldPos - camPos;
}

[Geometry Shader]

[Fragment Shader]

vec2 SphereMap( const in vec3 U, const in vec3 N )
{
	vec3 R;
	R = reflect( U, N );
	R.z += 1.0;
	R = normalize(R);
	return R.xy*0.5 + 0.5;
}

//	Texture uniforms
uniform sampler2D tex01;
uniform sampler2D tex02;
uniform sampler2D tex03;

//	Light color uniforms
uniform vec4 ambientLightColor;	//	Ambient light
uniform vec4 lightColors[4];

void main()
{
	//	Get texture information
	vec4 texture01 = texture2D( tex01, texCoord ).rgba;
	
	//	Alpha test
	if( texture01.a < 0.2 )
		discard;
	//	Use normal map's normals for the sphere map coordinates. lame but quick and good looking :)
	vec3 normal2 = texture2D(tex03, texCoord) * vec3(2.0) - vec3(1.0);
	vec2 texCoordSphere = SphereMap( normalize( viewVec ), normal2 );
	
	vec4 texture02 = texture2D( tex02, texCoordSphere ).rgba;
	
	vec4 finalColor = ambientLightColor;
	
	for( int i = 0; i < numLights; i++ )
	{
		float NdotL = max( dot( normal, lightVec[i] ), 0.0 );
		if( NdotL > 0.0 )
			finalColor += NdotL*lightColors[i];
	}

	gl_FragColor = finalColor * texture01 + texture02*0.2;
}