varying vec2 texCoord0;
varying vec2 texCoord1;
varying vec2 texCoord2;

varying float weight;
varying float acosT;
varying float asinT;

[Vertex Shader]

uniform vec3 sizes;
uniform mat4 worldMatrix;
uniform mat4 viewMatrix;
uniform mat4 projectionMatrix;

uniform float time;

void main()
{			
	//  Calculate World x View Matrix
	mat4 worldViewMatrix = viewMatrix * worldMatrix;
	//  Calculate Projection x WorldView Matrix
	mat4 worldViewProjectionMatrix = projectionMatrix * worldViewMatrix;
	//  Transform Vertex
	gl_Position = worldViewProjectionMatrix * gl_Vertex;
	
	//	Texture Coordinates for the day only cloud texture (moving slower and higher)
	texCoord0 = vec2( gl_Vertex.x / (sizes.x/5) + time / 4.0, gl_Vertex.z / (sizes.z/5) );
	
	//	Texture Coordinates for the day+night cloud texture (moving faster and lower)
	texCoord1 = vec2( gl_Vertex.x / (sizes.x/2) + time , gl_Vertex.z / (sizes.z/2) );
	
	//	Texture Coordinates for s night stars texture (not moving)
	texCoord2 = vec2( gl_Vertex.x / (sizes.x/8), gl_Vertex.z / (sizes.x/8) );
	
	//	Calculate a weight depending the height
	weight = clamp( gl_Vertex.y / sizes.y, 0.0, 1.0 );
	
	//	Send processed cosine ans sine to pixel shader
	acosT = abs(cos(time));
	asinT = abs(sin(time));
}

[Geometry Shader]

[Fragment Shader]

uniform sampler2D tex01;
uniform sampler2D tex02;
uniform sampler2D tex03;

void main()
{
	//	Sky Color
	vec4 sky_color = vec4( 0.5, 0.5, 0.0, 0.0 );
	
	//	2 clouds + 1 stars textures
	vec4 cloud0 = texture2D( tex01, texCoord0 ).rgba;
	vec4 cloud1 = texture2D( tex02, texCoord1 ).rgba;
	vec4 stars  = texture2D( tex03, texCoord2 ).rgba;
	
	//	Add the two cloud textures
	vec4 clouds = (cloud0*0.5 + cloud1);

	vec4 color = vec4( 1.0 - clouds.x, 1.0 - clouds.y, 1.0 - clouds.z, 0 );
	
	//	Calculate night Color
	vec4 nightColor = pow( acosT, 4.0 ) * (cloud1/8.0 + stars) * weight;	
	//	Calculate Day Color
	vec4 dayColor   = pow( asinT, 4.0 ) * (1.0 - (color * sky_color) * weight );
	
	//	Calculate Final Color
	gl_FragColor = nightColor + dayColor;
	
}