//	Varying variables
varying vec2 texCoord;	//	texCoords
varying vec3 normal;	//	normal
varying vec3 lightVec[4];		//	Light Vectors ( 1 - 4 Lights )
varying float weight;

//	Shared uniforms
uniform int numLights;		//	Number of lights

[Vertex Shader]

//	Transformation uniforms
uniform mat4 worldMatrix;		//	world matrix
uniform mat4 viewMatrix;		//	view/camera matrix
uniform mat4 projectionMatrix;	//	projection matrix

//	Lighting uniforms
uniform vec3 lightPositions[4];	//	Light positions

void main()
{
//	mat4 scale = mat4( 1, 0, 0, 0 ,   0, -1, 0, 0,   0, 0, 1, 0,   0, 0, 0, 1 );
	
	//  Calculate World x View Matrix
	mat4 worldViewMatrix = viewMatrix * worldMatrix;
//	worldViewMatrix = worldViewMatrix * scale;
	//  Calculate Projection x WorldView Matrix
	mat4 worldViewProjectionMatrix = projectionMatrix * worldViewMatrix;
	//  Transform Vertex
	gl_Position = worldViewProjectionMatrix * gl_Vertex;
	//  Pass Texture coordinates to pixel shader
	texCoord = gl_MultiTexCoord0;
	
	//  Vertex World position
	vec3 worldPos = worldMatrix * gl_Vertex;
	
	//	Loop through all lights and calculate light vectors
	for( int i = 0; i < numLights; i++ )
	{
		//  Calculate Light[i] vector in world position
		lightVec[i] = normalize( lightPositions[i] - worldPos );
	}

	//  Calculate normal
	normal = normalize( (mat3)worldMatrix * gl_Normal );
	
	weight = worldPos.y/120;
}

[Geometry Shader]

[Fragment Shader]

//	Texture uniforms
uniform sampler2D tex01;
uniform sampler2D tex02;

//	Light color uniforms
uniform vec4 ambientLightColor;	//	Ambient light
uniform vec4 lightColors[4];

void main()
{
	//	Get texture information
	vec4 texture01 = texture2D( tex01, texCoord ).rgba;
	//	Get texture information
	vec4 texture02 = texture2D( tex02, texCoord/4 ).rgba;
	
	vec4 finalColor = ambientLightColor;
	
	for( int i = 0; i < numLights; i++ )
	{
		float NdotL = max( dot( normal, lightVec[i] ), 0.0 );
		if( NdotL > 0.0 )
			finalColor += NdotL*lightColors[i];
	}

	gl_FragColor = finalColor*texture01*(1-weight) + finalColor*texture02*weight;
}